@@ -1,4 +1,5 @@
-from enum import Enum
+from collections.abc import Mapping
+from enum import StrEnum
 from typing import Any, Optional
 
 from pydantic import BaseModel, ConfigDict, field_validator
@@ -15,24 +16,25 @@ from dify_plugin.entities.model.message import (
 )
 
 
-class PluginInvokeType(Enum):
+class PluginInvokeType(StrEnum):
     Tool = "tool"
     Model = "model"
     Endpoint = "endpoint"
     Agent = "agent_strategy"
+    OAuth = "oauth"
 
 
-class AgentActions(Enum):
+class AgentActions(StrEnum):
     InvokeAgentStrategy = "invoke_agent_strategy"
 
 
-class ToolActions(Enum):
+class ToolActions(StrEnum):
     ValidateCredentials = "validate_tool_credentials"
     InvokeTool = "invoke_tool"
     GetToolRuntimeParameters = "get_tool_runtime_parameters"
 
 
-class ModelActions(Enum):
+class ModelActions(StrEnum):
     ValidateProviderCredentials = "validate_provider_credentials"
     ValidateModelCredentials = "validate_model_credentials"
     InvokeLLM = "invoke_llm"
@@ -47,10 +49,19 @@ class ModelActions(Enum):
     GetAIModelSchemas = "get_ai_model_schemas"
 
 
-class EndpointActions(Enum):
+class EndpointActions(StrEnum):
     InvokeEndpoint = "invoke_endpoint"
 
 
+class OAuthActions(StrEnum):
+    GetAuthorizationUrl = "get_authorization_url"
+    GetCredentials = "get_credentials"
+
+
+# merge all the access actions
+PluginAccessAction = AgentActions | ToolActions | ModelActions | EndpointActions
+
+
 class PluginAccessRequest(BaseModel):
     type: PluginInvokeType
     user_id: str
@@ -109,6 +120,9 @@ class PromptMessageMixin(BaseModel):
             raise ValueError("prompt_messages must be a list")
 
         for i in range(len(v)):
+            if isinstance(v[i], PromptMessage):
+                continue
+
             if v[i]["role"] == PromptMessageRole.USER.value:
                 v[i] = UserPromptMessage(**v[i])
             elif v[i]["role"] == PromptMessageRole.ASSISTANT.value:
@@ -220,3 +234,18 @@ class EndpointInvokeRequest(BaseModel):
     action: EndpointActions = EndpointActions.InvokeEndpoint
     settings: dict
     raw_http_request: str
+
+
+class OAuthGetAuthorizationUrlRequest(BaseModel):
+    type: PluginInvokeType = PluginInvokeType.OAuth
+    action: OAuthActions = OAuthActions.GetAuthorizationUrl
+    provider: str
+    system_credentials: Mapping[str, Any]
+
+
+class OAuthGetCredentialsRequest(BaseModel):
+    type: PluginInvokeType = PluginInvokeType.OAuth
+    action: OAuthActions = OAuthActions.GetCredentials
+    provider: str
+    system_credentials: Mapping[str, Any]
+    raw_http_request: str
