@@ -20,6 +20,8 @@ from dify_plugin.core.entities.plugin.request import (
     ModelInvokeTTSRequest,
     ModelValidateModelCredentialsRequest,
     ModelValidateProviderCredentialsRequest,
+    OAuthGetAuthorizationUrlRequest,
+    OAuthGetCredentialsRequest,
     ToolGetRuntimeParametersRequest,
     ToolInvokeRequest,
     ToolValidateCredentialsRequest,
@@ -27,6 +29,7 @@ from dify_plugin.core.entities.plugin.request import (
 from dify_plugin.core.plugin_registration import PluginRegistration
 from dify_plugin.core.runtime import Session
 from dify_plugin.core.utils.http_parser import parse_raw_request
+from dify_plugin.entities.agent import AgentRuntime
 from dify_plugin.entities.tool import ToolRuntime
 from dify_plugin.interfaces.endpoint import Endpoint
 from dify_plugin.interfaces.model.ai_model import AIModel
@@ -36,6 +39,8 @@ from dify_plugin.interfaces.model.rerank_model import RerankModel
 from dify_plugin.interfaces.model.speech2text_model import Speech2TextModel
 from dify_plugin.interfaces.model.text_embedding_model import TextEmbeddingModel
 from dify_plugin.interfaces.model.tts_model import TTSModel
+from dify_plugin.interfaces.tool import ToolProvider
+from dify_plugin.protocol.oauth import OAuthProviderProtocol
 
 
 class PluginExecutor:
@@ -82,7 +87,12 @@ class PluginExecutor:
                 f"Agent `{request.agent_strategy}` not found for provider `{request.agent_strategy_provider}`"
             )
 
-        agent = agent_cls(session=session)
+        agent = agent_cls(
+            runtime=AgentRuntime(
+                user_id=request.user_id,
+            ),
+            session=session,
+        )
         yield from agent.invoke(request.agent_strategy_params)
 
     def get_tool_runtime_parameters(self, session: Session, data: ToolGetRuntimeParametersRequest):
@@ -281,7 +291,7 @@ class PluginExecutor:
         except ValueError as e:
             response = Response(str(e), status=404)
         except Exception as e:
-            response = Response(f"Internal Server Error: {str(e)}", status=500)
+            response = Response(f"Internal Server Error: {e!s}", status=500)
 
         # check if response is a generator
         if isinstance(response.response, Generator):
@@ -315,3 +325,29 @@ class PluginExecutor:
                         result["result"] += binascii.hexlify(chunk.encode("utf-8")).decode()
 
             yield result
+
+    def _get_oauth_provider_instance(self, provider: str) -> OAuthProviderProtocol:
+        provider_cls = self.registration.get_supported_oauth_provider_cls(provider)
+        if provider_cls is None:
+            raise ValueError(f"Provider `{provider}` does not support OAuth")
+
+        if provider_cls == ToolProvider:
+            return provider_cls()
+
+        raise ValueError(f"Provider `{provider}` does not support OAuth")
+
+    def get_oauth_authorization_url(self, session: Session, data: OAuthGetAuthorizationUrlRequest):
+        provider_instance = self._get_oauth_provider_instance(data.provider)
+
+        return {
+            "authorization_url": provider_instance.oauth_get_authorization_url(data.system_credentials),
+        }
+
+    def get_oauth_credentials(self, session: Session, data: OAuthGetCredentialsRequest):
+        provider_instance = self._get_oauth_provider_instance(data.provider)
+        bytes_data = binascii.unhexlify(data.raw_http_request)
+        request = parse_raw_request(bytes_data)
+
+        return {
+            "credentials": provider_instance.oauth_get_credentials(data.system_credentials, request),
+        }
