@@ -1,13 +1,12 @@
-import json
 import uuid
 from abc import ABC
-from collections.abc import Generator
+from collections.abc import Generator, Mapping
 from concurrent.futures import ThreadPoolExecutor
 from enum import Enum
-from typing import Generic, Optional, TypeVar, Union
+from typing import Any, Generic, Optional, TypeVar, Union
 
 import httpx
-from pydantic import BaseModel
+from pydantic import BaseModel, TypeAdapter
 from yarl import URL
 
 from dify_plugin.config.config import InstallMethod
@@ -46,6 +45,7 @@ class ModelInvocations:
 
 class AppInvocations:
     def __init__(self, session: "Session"):
+        from dify_plugin.invocations.app import FetchAppInvocation
         from dify_plugin.invocations.app.chat import ChatAppInvocation
         from dify_plugin.invocations.app.completion import CompletionAppInvocation
         from dify_plugin.invocations.app.workflow import WorkflowAppInvocation
@@ -53,6 +53,10 @@ class AppInvocations:
         self.chat = ChatAppInvocation(session)
         self.completion = CompletionAppInvocation(session)
         self.workflow = WorkflowAppInvocation(session)
+        self.fetch_app_invocation = FetchAppInvocation(session)
+
+    def fetch_app(self, app_id: str) -> Mapping:
+        return self.fetch_app_invocation.get(app_id)
 
 
 class WorkflowNodeInvocations:
@@ -219,7 +223,7 @@ class BackwardsInvocation(Generic[T], ABC):
             try:
                 yield data_type(**event.data)
             except Exception as e:
-                raise Exception(f"Failed to parse response: {str(e)}") from e
+                raise Exception(f"Failed to parse response: {e!s}") from e
 
     def _http_backwards_invoke(
         self,
@@ -271,7 +275,7 @@ class BackwardsInvocation(Generic[T], ABC):
                     if not line:
                         continue
 
-                    data = json.loads(line)
+                    data = TypeAdapter(dict[str, Any]).validate_json(line)
                     yield PluginInStreamBase(
                         session_id=data["session_id"],
                         event=PluginInStreamEvent.value_of(data["event"]),
