@@ -2,18 +2,18 @@ import errno
 import logging
 import os
 import signal
-from threading import Lock
-import time
 import socket as native_socket
-from collections.abc import Generator
-from json import loads
-from typing import Callable, Optional
+import time
+from collections.abc import Callable, Generator
+from threading import Lock
+from typing import Any, Optional
 
+from gevent import sleep
+from gevent import socket as gevent_socket
 from gevent.select import select
-from gevent import socket as gevent_socket, sleep
+from pydantic import TypeAdapter
 
 from dify_plugin.core.entities.message import InitializeMessage
-
 from dify_plugin.core.entities.plugin.io import (
     PluginInStream,
     PluginInStreamEvent,
@@ -37,6 +37,8 @@ class TCPReaderWriter(RequestReader, ResponseWriter):
         """
         Initialize the TCPStream and connect to the target, raise exception if connection failed
         """
+        super().__init__()
+
         self.host = host
         self.port = port
         self.key = key
@@ -74,8 +76,7 @@ class TCPReaderWriter(RequestReader, ResponseWriter):
         """
         Receive data from the socket
         """
-        with self.opt_lock:
-            return self.sock.recv(size)
+        return self.sock.recv(size)
 
     def write(self, data: str):
         if not self.alive:
@@ -188,9 +189,13 @@ class TCPReaderWriter(RequestReader, ResponseWriter):
             lines = lines[:-1]
             for line in lines:
                 try:
-                    data = loads(line)
+                    data = TypeAdapter(dict[str, Any]).validate_json(line)
                     chunk = PluginInStream(
                         session_id=data["session_id"],
+                        conversation_id=data.get("conversation_id"),
+                        message_id=data.get("message_id"),
+                        app_id=data.get("app_id"),
+                        endpoint_id=data.get("endpoint_id"),
                         event=PluginInStreamEvent.value_of(data["event"]),
                         data=data["data"],
                         reader=self,
