@@ -13,58 +13,12 @@ from pydantic import (
     model_validator,
 )
 
+from dify_plugin.core.documentation.schema_doc import docs
 from dify_plugin.core.utils.yaml_loader import load_yaml_file
 from dify_plugin.entities import I18nObject
 from dify_plugin.entities.model.message import PromptMessageTool
-
-
-class LogMetadata(str, Enum):
-    STARTED_AT = "started_at"
-    FINISHED_AT = "finished_at"
-    ELAPSED_TIME = "elapsed_time"
-    TOTAL_PRICE = "total_price"
-    TOTAL_TOKENS = "total_tokens"
-    PROVIDER = "provider"
-    CURRENCY = "currency"
-
-
-class CommonParameterType(Enum):
-    SECRET_INPUT = "secret-input"
-    TEXT_INPUT = "text-input"
-    SELECT = "select"
-    STRING = "string"
-    NUMBER = "number"
-    FILE = "file"
-    FILES = "files"
-    BOOLEAN = "boolean"
-    APP_SELECTOR = "app-selector"
-    MODEL_SELECTOR = "model-selector"
-    # TOOL_SELECTOR = "tool-selector"
-    TOOLS_SELECTOR = "array[tools]"
-
-
-class AppSelectorScope(Enum):
-    ALL = "all"
-    CHAT = "chat"
-    WORKFLOW = "workflow"
-    COMPLETION = "completion"
-
-
-class ModelConfigScope(Enum):
-    LLM = "llm"
-    TEXT_EMBEDDING = "text-embedding"
-    RERANK = "rerank"
-    TTS = "tts"
-    SPEECH2TEXT = "speech2text"
-    MODERATION = "moderation"
-    VISION = "vision"
-
-
-class ToolSelectorScope(Enum):
-    ALL = "all"
-    PLUGIN = "plugin"
-    API = "api"
-    WORKFLOW = "workflow"
+from dify_plugin.entities.oauth import OAuthSchema
+from dify_plugin.entities.provider_config import CommonParameterType, LogMetadata, ProviderConfig
 
 
 class ToolRuntime(BaseModel):
@@ -81,7 +35,7 @@ class ToolInvokeMessage(BaseModel):
             return {"text": self.text}
 
     class JsonMessage(BaseModel):
-        json_object: dict
+        json_object: Mapping | list
 
         def to_dict(self):
             return {"json_object": self.json_object}
@@ -129,6 +83,32 @@ class ToolInvokeMessage(BaseModel):
         data: Mapping[str, Any] = Field(..., description="Detailed log data")
         metadata: Optional[Mapping[LogMetadata, Any]] = Field(default=None, description="The metadata of the log")
 
+    class RetrieverResourceMessage(BaseModel):
+        class RetrieverResource(BaseModel):
+            """
+            Model class for retriever resource.
+            """
+
+            position: Optional[int] = None
+            dataset_id: Optional[str] = None
+            dataset_name: Optional[str] = None
+            document_id: Optional[str] = None
+            document_name: Optional[str] = None
+            data_source_type: Optional[str] = None
+            segment_id: Optional[str] = None
+            retriever_from: Optional[str] = None
+            score: Optional[float] = None
+            hit_count: Optional[int] = None
+            word_count: Optional[int] = None
+            segment_position: Optional[int] = None
+            index_node_hash: Optional[str] = None
+            content: Optional[str] = None
+            page: Optional[int] = None
+            doc_metadata: Optional[dict] = None
+
+        retriever_resources: list[RetrieverResource] = Field(..., description="retriever resources")
+        context: str = Field(..., description="context")
+
     class MessageType(Enum):
         TEXT = "text"
         FILE = "file"
@@ -140,11 +120,21 @@ class ToolInvokeMessage(BaseModel):
         VARIABLE = "variable"
         BLOB_CHUNK = "blob_chunk"
         LOG = "log"
+        RETRIEVER_RESOURCES = "retriever_resources"
 
     type: MessageType
     # TODO: pydantic will validate and construct the message one by one, until it encounters a correct type
     # we need to optimize the construction process
-    message: TextMessage | JsonMessage | VariableMessage | BlobMessage | BlobChunkMessage | LogMessage | None
+    message: (
+        TextMessage
+        | JsonMessage
+        | VariableMessage
+        | BlobMessage
+        | BlobChunkMessage
+        | LogMessage
+        | RetrieverResourceMessage
+        | None
+    )
     meta: Optional[dict] = None
 
     @field_validator("message", mode="before")
@@ -170,12 +160,18 @@ class ToolInvokeMessage(BaseModel):
         return v
 
 
+@docs(
+    description="The identity of the tool",
+)
 class ToolIdentity(BaseModel):
     author: str = Field(..., description="The author of the tool")
     name: str = Field(..., description="The name of the tool")
     label: I18nObject = Field(..., description="The label of the tool")
 
 
+@docs(
+    description="The option of the tool parameter",
+)
 class ToolParameterOption(BaseModel):
     value: str = Field(..., description="The value of the option")
     label: I18nObject = Field(..., description="The label of the option")
@@ -189,6 +185,9 @@ class ToolParameterOption(BaseModel):
             return value
 
 
+@docs(
+    description="The auto generate of the parameter",
+)
 class ParameterAutoGenerate(BaseModel):
     class Type(StrEnum):
         PROMPT_INSTRUCTION = "prompt_instruction"
@@ -196,10 +195,16 @@ class ParameterAutoGenerate(BaseModel):
     type: Type
 
 
+@docs(
+    description="The template of the parameter",
+)
 class ParameterTemplate(BaseModel):
     enabled: bool = Field(..., description="Whether the parameter is jinja enabled")
 
 
+@docs(
+    description="The type of the parameter",
+)
 class ToolParameter(BaseModel):
     class ToolParameterType(str, Enum):
         STRING = CommonParameterType.STRING.value
@@ -212,6 +217,7 @@ class ToolParameter(BaseModel):
         MODEL_SELECTOR = CommonParameterType.MODEL_SELECTOR.value
         APP_SELECTOR = CommonParameterType.APP_SELECTOR.value
         # TOOL_SELECTOR = CommonParameterType.TOOL_SELECTOR.value
+        ANY = CommonParameterType.ANY.value
 
     class ToolParameterForm(Enum):
         SCHEMA = "schema"  # should be set while adding tool
@@ -237,11 +243,18 @@ class ToolParameter(BaseModel):
     options: Optional[list[ToolParameterOption]] = None
 
 
+@docs(
+    description="The description of the tool",
+)
 class ToolDescription(BaseModel):
     human: I18nObject = Field(..., description="The description presented to the user")
     llm: str = Field(..., description="The description presented to the LLM")
 
 
+@docs(
+    name="ToolExtra",
+    description="The extra of the tool",
+)
 class ToolConfigurationExtra(BaseModel):
     class Python(BaseModel):
         source: str
@@ -249,6 +262,10 @@ class ToolConfigurationExtra(BaseModel):
     python: Python
 
 
+@docs(
+    name="Tool",
+    description="The manifest of the tool",
+)
 class ToolConfiguration(BaseModel):
     identity: ToolIdentity
     parameters: list[ToolParameter] = Field(default=[], description="The parameters of the tool")
@@ -258,6 +275,9 @@ class ToolConfiguration(BaseModel):
     output_schema: Optional[Mapping[str, Any]] = None
 
 
+@docs(
+    description="The label of the tool",
+)
 class ToolLabelEnum(Enum):
     SEARCH = "search"
     IMAGE = "image"
@@ -277,47 +297,9 @@ class ToolLabelEnum(Enum):
     OTHER = "other"
 
 
-class ToolCredentialsOption(BaseModel):
-    value: str = Field(..., description="The value of the option")
-    label: I18nObject = Field(..., description="The label of the option")
-
-
-class ProviderConfig(BaseModel):
-    class Config(Enum):
-        SECRET_INPUT = CommonParameterType.SECRET_INPUT.value
-        TEXT_INPUT = CommonParameterType.TEXT_INPUT.value
-        SELECT = CommonParameterType.SELECT.value
-        BOOLEAN = CommonParameterType.BOOLEAN.value
-        MODEL_SELECTOR = CommonParameterType.MODEL_SELECTOR.value
-        APP_SELECTOR = CommonParameterType.APP_SELECTOR.value
-        # TOOL_SELECTOR = CommonParameterType.TOOL_SELECTOR.value
-        TOOLS_SELECTOR = CommonParameterType.TOOLS_SELECTOR.value
-
-        @classmethod
-        def value_of(cls, value: str) -> "ProviderConfig.Config":
-            """
-            Get value of given mode.
-
-            :param value: mode value
-            :return: mode
-            """
-            for mode in cls:
-                if mode.value == value:
-                    return mode
-            raise ValueError(f"invalid mode value {value}")
-
-    name: str = Field(..., description="The name of the credentials")
-    type: Config = Field(..., description="The type of the credentials")
-    scope: str | None = None
-    required: bool = False
-    default: Optional[Union[int, float, str]] = None
-    options: Optional[list[ToolCredentialsOption]] = None
-    label: I18nObject
-    help: Optional[I18nObject] = None
-    url: Optional[str] = None
-    placeholder: Optional[I18nObject] = None
-
-
+@docs(
+    description="The identity of the tool provider",
+)
 class ToolProviderIdentity(BaseModel):
     author: str = Field(..., description="The author of the tool")
     name: str = Field(..., description="The name of the tool")
@@ -330,6 +312,10 @@ class ToolProviderIdentity(BaseModel):
     )
 
 
+@docs(
+    name="ToolProviderExtra",
+    description="The extra of the tool provider",
+)
 class ToolProviderConfigurationExtra(BaseModel):
     class Python(BaseModel):
         source: str
@@ -337,6 +323,11 @@ class ToolProviderConfigurationExtra(BaseModel):
     python: Python
 
 
+@docs(
+    name="ToolProvider",
+    description="The Manifest of the tool provider",
+    outside_reference_fields={"tools": ToolConfiguration},
+)
 class ToolProviderConfiguration(BaseModel):
     identity: ToolProviderIdentity
     credentials_schema: list[ProviderConfig] = Field(
@@ -344,6 +335,10 @@ class ToolProviderConfiguration(BaseModel):
         alias="credentials_for_provider",
         description="The credentials schema of the tool provider",
     )
+    oauth_schema: Optional[OAuthSchema] = Field(
+        default=None,
+        description="The OAuth schema of the tool provider if OAuth is supported",
+    )
     tools: list[ToolConfiguration] = Field(default=[], description="The tools of the tool provider")
     extra: ToolProviderConfigurationExtra
 
@@ -384,7 +379,7 @@ class ToolProviderConfiguration(BaseModel):
                     )
                 )
             except Exception as e:
-                raise ValueError(f"Error loading tool configuration: {str(e)}") from e
+                raise ValueError(f"Error loading tool configuration: {e!s}") from e
 
         return tools
 
