@@ -1,12 +1,12 @@
 import logging
 from abc import abstractmethod
 from collections.abc import Generator, Mapping
-from typing import Any, Optional, Union
+from typing import Any, Optional, Union, final
 
 from pydantic import BaseModel, ConfigDict, Field, ValidationInfo, field_validator
 
 from dify_plugin.core.runtime import Session
-from dify_plugin.entities.agent import AgentInvokeMessage
+from dify_plugin.entities.agent import AgentInvokeMessage, AgentRuntime
 from dify_plugin.entities.model import AIModelEntity, ModelPropertyKey
 from dify_plugin.entities.model.llm import LLMModelConfig, LLMUsage
 from dify_plugin.entities.model.message import (
@@ -28,7 +28,7 @@ class AgentToolIdentity(ToolIdentity):
 
 class AgentModelConfig(LLMModelConfig):
     entity: AIModelEntity | None = None
-    history_prompt_messages: list[PromptMessage] = []
+    history_prompt_messages: list[PromptMessage] = Field(default_factory=list)
 
     @field_validator("history_prompt_messages", mode="before")
     @classmethod
@@ -149,10 +149,19 @@ class AgentProvider(ToolProvider):
 
 
 class AgentStrategy(ToolLike[AgentInvokeMessage]):
+    @final
     def __init__(
         self,
+        runtime: AgentRuntime,
         session: Session,
     ):
+        """
+        Initialize the agent strategy
+
+        NOTE:
+        - This method has been marked as final, DO NOT OVERRIDE IT.
+        """
+        self.runtime = runtime
         self.session = session
         self.response_type = AgentInvokeMessage
 
@@ -277,6 +286,11 @@ class AgentStrategy(ToolLike[AgentInvokeMessage]):
                 ToolParameter.ToolParameterType.FILES,
             }:
                 continue
+            if parameter.type in {
+                ToolParameter.ToolParameterType.SELECT,
+                ToolParameter.ToolParameterType.SECRET_INPUT,
+            }:
+                parameter_type = ToolParameter.ToolParameterType.STRING
             enum = []
             if parameter.type == ToolParameter.ToolParameterType.SELECT:
                 enum = [option.value for option in parameter.options] if parameter.options else []
@@ -311,6 +325,11 @@ class AgentStrategy(ToolLike[AgentInvokeMessage]):
                 ToolParameter.ToolParameterType.FILES,
             }:
                 continue
+            if parameter.type in {
+                ToolParameter.ToolParameterType.SELECT,
+                ToolParameter.ToolParameterType.SECRET_INPUT,
+            }:
+                parameter_type = ToolParameter.ToolParameterType.STRING
             enum = []
             if parameter.type == ToolParameter.ToolParameterType.SELECT:
                 enum = [option.value for option in parameter.options] if parameter.options else []
