@@ -1,7 +1,8 @@
 import decimal
+import socket
 from abc import ABC, abstractmethod
 from collections.abc import Mapping
-from typing import Optional
+from typing import Optional, final
 
 import gevent.socket
 from pydantic import ConfigDict
@@ -18,8 +19,6 @@ from dify_plugin.entities.model import (
 )
 from dify_plugin.errors.model import InvokeAuthorizationError, InvokeError
 
-import socket
-
 if socket.socket is gevent.socket.socket:
     import gevent.threadpool
 
@@ -38,7 +37,14 @@ class AIModel(ABC):
     # pydantic configs
     model_config = ConfigDict(protected_namespaces=())
 
+    @final
     def __init__(self, model_schemas: list[AIModelEntity]) -> None:
+        """
+        Initialize the model
+
+        NOTE:
+        - This method has been marked as final, DO NOT OVERRIDE IT.
+        """
         self.model_schemas = [
             model_schema for model_schema in model_schemas if model_schema.model_type == self.model_type
         ]
@@ -84,9 +90,9 @@ class AIModel(ABC):
                         "please check and try again. "
                     )
 
-                return invoke_error(description=f"[{provider_name}] {invoke_error.description}, {str(error)}")
+                return invoke_error(description=f"[{provider_name}] {invoke_error.description}, {error!s}")
 
-        return InvokeError(description=f"[{provider_name}] Error: {str(error)}")
+        return InvokeError(description=f"[{provider_name}] Error: {error!s}")
 
     def get_price(self, model: str, credentials: dict, price_type: PriceType, tokens: int) -> PriceInfo:
         """
@@ -271,11 +277,11 @@ class AIModel(ABC):
         if len(text) >= 100000:
             return len(text)
 
-        import tiktoken
-
         # check if gevent is patched to main thread
         import socket
 
+        import tiktoken
+
         if socket.socket is gevent.socket.socket:
             # using gevent real thread to avoid blocking main thread
             result = threadpool.spawn(lambda: len(tiktoken.encoding_for_model("gpt2").encode(text)))
