@@ -1,13 +1,14 @@
 import codecs
 import json
-
 import logging
+import uuid
 from collections.abc import Generator
 from decimal import Decimal
-from typing import Optional, Union, cast
+from typing import Any, Optional, Union, cast
 from urllib.parse import urljoin
 
 import requests
+from pydantic import TypeAdapter, ValidationError
 
 from dify_plugin.entities import I18nObject
 from dify_plugin.entities.model import (
@@ -46,6 +47,58 @@ from dify_plugin.interfaces.model.openai_compatible.common import _CommonOaiApiC
 logger = logging.getLogger(__name__)
 
 
+def _gen_tool_call_id() -> str:
+    return f"chatcmpl-tool-{uuid.uuid4().hex!s}"
+
+
+def _increase_tool_call(
+    new_tool_calls: list[AssistantPromptMessage.ToolCall], existing_tools_calls: list[AssistantPromptMessage.ToolCall]
+):
+    """
+    Merge incremental tool call updates into existing tool calls.
+
+    :param new_tool_calls: List of new tool call deltas to be merged.
+    :param existing_tools_calls: List of existing tool calls to be modified IN-PLACE.
+    """
+
+    def get_tool_call(tool_call_id: str):
+        """
+        Get or create a tool call by ID
+
+        :param tool_call_id: tool call ID
+        :return: existing or new tool call
+        """
+        if not tool_call_id:
+            return existing_tools_calls[-1]
+
+        _tool_call = next((_tool_call for _tool_call in existing_tools_calls if _tool_call.id == tool_call_id), None)
+        if _tool_call is None:
+            _tool_call = AssistantPromptMessage.ToolCall(
+                id=tool_call_id,
+                type="function",
+                function=AssistantPromptMessage.ToolCall.ToolCallFunction(name="", arguments=""),
+            )
+            existing_tools_calls.append(_tool_call)
+
+        return _tool_call
+
+    for new_tool_call in new_tool_calls:
+        # generate ID for tool calls with function name but no ID to track them
+        if new_tool_call.function.name and not new_tool_call.id:
+            new_tool_call.id = _gen_tool_call_id()
+        # get tool call
+        tool_call = get_tool_call(new_tool_call.id)
+        # update tool call
+        if new_tool_call.id:
+            tool_call.id = new_tool_call.id
+        if new_tool_call.type:
+            tool_call.type = new_tool_call.type
+        if new_tool_call.function.name:
+            tool_call.function.name = new_tool_call.function.name
+        if new_tool_call.function.arguments:
+            tool_call.function.arguments += new_tool_call.function.arguments
+
+
 class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
     """
     Model class for OpenAI large language model.
@@ -164,8 +217,8 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
 
             try:
                 json_result = response.json()
-            except json.JSONDecodeError as e:
-                raise CredentialsValidateFailedError("Credentials validation failed: JSON decode error")
+            except json.JSONDecodeError:
+                raise CredentialsValidateFailedError("Credentials validation failed: JSON decode error") from None
 
             if completion_type is LLMMode.CHAT and json_result.get("object", "") == "":
                 json_result["object"] = "chat.completion"
@@ -187,7 +240,7 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
         except CredentialsValidateFailedError:
             raise
         except Exception as ex:
-            raise CredentialsValidateFailedError(f"An error occurred during credentials validation: {str(ex)}")
+            raise CredentialsValidateFailedError(f"An error occurred during credentials validation: {ex!s}") from ex
 
     def get_customizable_model_schema(self, model: str, credentials: dict) -> AIModelEntity:
         """
@@ -357,7 +410,7 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
                 if not json_schema:
                     raise ValueError("Must define JSON Schema when the response format is json_schema")
                 try:
-                    schema = json.loads(json_schema)
+                    schema = TypeAdapter(dict[str, Any]).validate_json(json_schema)
                 except Exception as exc:
                     raise ValueError(f"not correct json_schema format: {json_schema}") from exc
                 model_parameters.pop("json_schema")
@@ -444,49 +497,9 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
 
         return LLMResultChunk(
             model=model,
-            prompt_messages=prompt_messages,
             delta=LLMResultChunkDelta(index=index, message=message, finish_reason=finish_reason, usage=usage),
         )
 
-    def _get_tool_call(self, tool_call_id: str, tools_calls: list[AssistantPromptMessage.ToolCall]):
-        """
-        Get or create a tool call by ID
-
-        :param tool_call_id: tool call ID
-        :param tools_calls: list of existing tool calls
-        :return: existing or new tool call, updated tools_calls
-        """
-        if not tool_call_id:
-            return tools_calls[-1], tools_calls
-
-        tool_call = next((tool_call for tool_call in tools_calls if tool_call.id == tool_call_id), None)
-        if tool_call is None:
-            tool_call = AssistantPromptMessage.ToolCall(
-                id=tool_call_id,
-                type="function",
-                function=AssistantPromptMessage.ToolCall.ToolCallFunction(name="", arguments=""),
-            )
-            tools_calls.append(tool_call)
-
-        return tool_call, tools_calls
-
-    def _increase_tool_call(
-        self, new_tool_calls: list[AssistantPromptMessage.ToolCall], tools_calls: list[AssistantPromptMessage.ToolCall]
-    ) -> list[AssistantPromptMessage.ToolCall]:
-        for new_tool_call in new_tool_calls:
-            # get tool call
-            tool_call, tools_calls = self._get_tool_call(new_tool_call.function.name, tools_calls)
-            # update tool call
-            if new_tool_call.id:
-                tool_call.id = new_tool_call.id
-            if new_tool_call.type:
-                tool_call.type = new_tool_call.type
-            if new_tool_call.function.name:
-                tool_call.function.name = new_tool_call.function.name
-            if new_tool_call.function.arguments:
-                tool_call.function.arguments += new_tool_call.function.arguments
-        return tools_calls
-
     def _handle_generate_stream_response(
         self, model: str, credentials: dict, response: requests.Response, prompt_messages: list[PromptMessage]
     ) -> Generator:
@@ -519,9 +532,9 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
                     continue
 
                 try:
-                    chunk_json: dict = json.loads(decoded_chunk)
+                    chunk_json: dict = TypeAdapter(dict[str, Any]).validate_json(decoded_chunk)
                 # stream ended
-                except json.JSONDecodeError as e:
+                except ValidationError:
                     yield self._create_final_llm_result_chunk(
                         index=chunk_index + 1,
                         message=AssistantPromptMessage(content=""),
@@ -537,7 +550,7 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
                 if chunk_json.get("error") and chunk_json.get("choices") is None:
                     raise ValueError(chunk_json.get("error"))
 
-                if chunk_json:
+                if chunk_json:  # noqa: SIM102
                     if u := chunk_json.get("usage"):
                         usage = u
                 if not chunk_json or len(chunk_json["choices"]) == 0:
@@ -568,7 +581,7 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
                     # extract tool calls from response
                     if assistant_message_tool_calls:
                         tool_calls = self._extract_response_tool_calls(assistant_message_tool_calls)
-                        tools_calls = self._increase_tool_call(tool_calls, tools_calls)
+                        _increase_tool_call(tool_calls, tools_calls)
 
                     if delta_content is None or delta_content == "":
                         continue
@@ -578,8 +591,6 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
                         content=delta_content,
                     )
 
-                    # reset tool calls
-                    tool_calls = []
                     full_assistant_content += delta_content
                 elif "text" in choice:
                     choice_text = choice.get("text", "")
@@ -594,7 +605,6 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
 
                 yield LLMResultChunk(
                     model=model,
-                    prompt_messages=prompt_messages,
                     delta=LLMResultChunkDelta(
                         index=chunk_index,
                         message=assistant_prompt_message,
@@ -606,7 +616,6 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
         if tools_calls:
             yield LLMResultChunk(
                 model=model,
-                prompt_messages=prompt_messages,
                 delta=LLMResultChunkDelta(
                     index=chunk_index,
                     message=AssistantPromptMessage(tool_calls=tools_calls, content=""),
@@ -678,7 +687,6 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
         result = LLMResult(
             id=message_id,
             model=response_json["model"],
-            prompt_messages=prompt_messages,
             message=assistant_message,
             usage=usage,
         )
@@ -898,6 +906,8 @@ class OAICompatLargeLanguageModel(_CommonOaiApiCompat, LargeLanguageModel):
         tool_calls = []
         if response_tool_calls:
             for response_tool_call in response_tool_calls:
+                if not response_tool_call.get("function"):
+                    continue
                 function = AssistantPromptMessage.ToolCall.ToolCallFunction(
                     name=response_tool_call.get("function", {}).get("name", ""),
                     arguments=response_tool_call.get("function", {}).get("arguments", ""),
