@@ -1,6 +1,8 @@
 from abc import ABC, abstractmethod
 from collections.abc import Generator, Mapping
-from typing import Any, Generic, Optional, Type, TypeVar
+from typing import Any, Generic, Optional, TypeVar, final
+
+from werkzeug import Request
 
 from dify_plugin.core.runtime import Session
 from dify_plugin.entities.agent import AgentInvokeMessage
@@ -13,7 +15,7 @@ T = TypeVar("T", bound=ToolInvokeMessage | AgentInvokeMessage)
 
 
 class ToolLike(ABC, Generic[T]):
-    response_type: Type[T]
+    response_type: type[T]
 
     ############################################################
     #            For plugin implementation use only            #
@@ -25,7 +27,7 @@ class ToolLike(ABC, Generic[T]):
             message=ToolInvokeMessage.TextMessage(text=text),
         )
 
-    def create_json_message(self, json: dict) -> T:
+    def create_json_message(self, json: Mapping | list) -> T:
         return self.response_type(
             type=ToolInvokeMessage.MessageType.JSON,
             message=ToolInvokeMessage.JsonMessage(json_object=json),
@@ -124,6 +126,22 @@ class ToolLike(ABC, Generic[T]):
             ),
         )
 
+    def create_retriever_resource_message(
+        self,
+        retriever_resources: list[ToolInvokeMessage.RetrieverResourceMessage.RetrieverResource],
+        context: str,
+    ) -> T:
+        """
+        create a retriever resource message
+        """
+        return self.response_type(
+            type=ToolInvokeMessage.MessageType.RETRIEVER_RESOURCES,
+            message=ToolInvokeMessage.RetrieverResourceMessage(
+                retriever_resources=retriever_resources,
+                context=context,
+            ),
+        )
+
     def finish_log_message(
         self,
         log: T,
@@ -206,24 +224,42 @@ class ToolLike(ABC, Generic[T]):
         return tool_parameters
 
 
-class ToolProvider(ABC):
+class ToolProvider:
     def validate_credentials(self, credentials: dict):
         return self._validate_credentials(credentials)
 
-    @abstractmethod
     def _validate_credentials(self, credentials: dict):
-        pass
+        raise NotImplementedError("This method should be implemented by a subclass")
+
+    def oauth_get_authorization_url(self, system_credentials: Mapping[str, Any]) -> str:
+        return self._oauth_get_authorization_url(system_credentials)
+
+    def _oauth_get_authorization_url(self, system_credentials: Mapping[str, Any]) -> str:
+        raise NotImplementedError("This method should be implemented by a subclass")
+
+    def oauth_get_credentials(self, system_credentials: Mapping[str, Any], request: Request) -> Mapping[str, Any]:
+        return self._oauth_get_credentials(system_credentials, request)
+
+    def _oauth_get_credentials(self, system_credentials: Mapping[str, Any], request: Request) -> Mapping[str, Any]:
+        raise NotImplementedError("This method should be implemented by a subclass")
 
 
 class Tool(ToolLike[ToolInvokeMessage]):
     runtime: ToolRuntime
     session: Session
 
+    @final
     def __init__(
         self,
         runtime: ToolRuntime,
         session: Session,
     ):
+        """
+        Initialize the tool
+
+        NOTE:
+        - This method has been marked as final, DO NOT OVERRIDE IT.
+        """
         self.runtime = runtime
         self.session = session
         self.response_type = ToolInvokeMessage
